import unittest
from unittest.mock import patch

import demo
import pg_util
import trials


class TestDemo(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        super(TestDemo, cls).setUpClass()
        TestDemo.dropdb()

    def tearDown(self):
        super(TestDemo, self).tearDown()
        TestDemo.dropdb()

    @staticmethod
    def dropdb():
        with pg_util.cursor("postgres") as cr:
            cr.execute(
                r"SELECT datname FROM pg_catalog.pg_database WHERE datname like 'demo\_%'"
            )
            dbs = cr.fetchall()
            for db in dbs:
                trials.drop(db[0])

    def test_demo_list(self):
        pg_util.createdb("demo_160_test")
        dbs = demo.demo_list()
        self.assertEqual(dbs, ["demo_160_test"])

    def test_demo_time(self):
        self.assertEqual(demo.demo_time("demo_160_1658240170"), 1658240170)

    @patch("demo.time.time")
    def test_demo_gc(self, time_mock):
        time_mock.return_value = 7200

        # This one must be dropped
        pg_util.createdb("demo_160_0000001800")
        # These two must remain
        pg_util.createdb("demo_160_0000005400")
        pg_util.createdb("demo_160_0000003600")

        demo.demo_gc()

        dbs = demo.demo_list(all_versions=True)
        self.assertFalse("demo_160_0000001800" in dbs)
        self.assertTrue("demo_160_0000003600" in dbs)
        self.assertTrue("demo_160_0000005400" in dbs)
